/**
 * 
 */
package org.q2l.smallab;

import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;

import processing.core.*;
//import codeanticode.glgraphics.*;

/**
 * @author Mike Edwards (mike@onearmedman.com)
 *
 */
public class PartsOfWholeDemo extends PApplet {

	/**
	 * 
	 */
	private static final long serialVersionUID = -651155362339430280L;
	private boolean positioned = false;

	public PFont numberFont;
	public Fraction firstFraction;
	public Fraction secondFraction;
	public FractionView firstFractionView;
	public FractionView secondFractionView;
	public int lcd;
	
	/**
	 * 
	 */
	public PartsOfWholeDemo() {
		super();
	}

	/* (non-Javadoc)
	 * @see processing.core.PApplet#setup()
	 */
	@Override
	public void setup() {
		/*
		 * add in player active and queue areas, animated feedback, sound, etc. 
		 */
		size(1024,768);
		
		this.smooth();
		
		this.numberFont = createFont("DejaVuSansMono.ttf", 25.0f);

		firstFraction = new Fraction(1,3);
		secondFraction = new Fraction(1,5);

		lcd = (int) (firstFraction.denominator() * secondFraction.denominator() / 
					Fraction.gcd(firstFraction.denominator(), secondFraction.denominator()));

		firstFractionView = new FractionView(this,width,height,firstFraction,lcd);
		firstFractionView.blockColor(0, 0xff, 0x98);
		// Claudio's green color #00ff98

		secondFractionView = new FractionView(this,width,height,secondFraction,lcd);
		secondFractionView.blockColor(0xff, 0x32, 0x99);
		// Claudio's pink color #ff3299

		firstFractionView.fraction = firstFraction;
		secondFractionView.fraction = secondFraction;
		
		this.noLoop();
	}
	
	
	/* (non-Javadoc)
	 * @see processing.core.PApplet#draw()
	 */
	@Override
	public void draw() {
		//autostarts the fullscreen app in the highest numbered display (e.g. projector, second-screen, etc.)
		if (!positioned)
		{
			positioned = true;
			GraphicsEnvironment environment = GraphicsEnvironment.getLocalGraphicsEnvironment();
			GraphicsDevice[] devices = environment.getScreenDevices();
			if (devices.length > 0)
			{
				Rectangle fullScreenRect = devices[devices.length - 1].getDefaultConfiguration().getBounds();
				this.frame.setBounds(fullScreenRect);
				this.frame.setLocation(fullScreenRect.x, fullScreenRect.y);
			}
		}

		this.background(51);
		
		textFont(numberFont);
		if (this.lcd == this.firstFraction.denominator() * this.firstFractionView.multiplier &&
				this.lcd == this.secondFraction.denominator() * this.secondFractionView.multiplier)
		{
			this.firstFractionView.common = true;
			this.secondFractionView.common = true;
		}
		else
		{
			this.firstFractionView.common = false;
			this.secondFractionView.common = false;
			this.firstFractionView.added = false;
			this.secondFractionView.added = false;
		}

		String equation;
		if (this.firstFractionView.added && this.secondFractionView.added)
		{
			Fraction sum = this.firstFraction.plus(this.secondFraction);
			int multiplier = 1;
			if (sum.denominator() != lcd)
			{
				multiplier = lcd/(int)sum.denominator();
			}
			equation = this.firstFraction.numerator() * this.firstFractionView.multiplier + "/" + this.firstFraction.denominator() * this.firstFractionView.multiplier + 
				" + " + 
				this.secondFraction.numerator() * this.secondFractionView.multiplier + "/" + this.secondFraction.denominator() * this.secondFractionView.multiplier + 
				" = " + 
				sum.numerator() * multiplier + "/" + sum.denominator() * multiplier;
			if (multiplier != 1)
			{
				equation += " = ?";
			}
		} 
		else
		{
			equation = this.firstFraction.numerator() * this.firstFractionView.multiplier + "/" + this.firstFraction.denominator() * this.firstFractionView.multiplier + 
			" + " + 
			this.secondFraction.numerator() * this.secondFractionView.multiplier + "/" + this.secondFraction.denominator() * this.secondFractionView.multiplier + 
			" = ?";
		}
		this.text(equation,width/2 - this.textWidth(equation)/2,height/2+FractionView.BLOCK_HEIGHT*1.5f);
		this.pushMatrix();
		this.translate(0, height);
		this.rotate(this.radians(180));
		this.text(equation,width/-2 - this.textWidth(equation)/2,height/2+FractionView.BLOCK_HEIGHT*1.5f);
		this.popMatrix();

		this.pushMatrix();
		this.translate(this.firstFractionView.renderWidth(), 300);
		this.rotate(PApplet.radians(180));
		firstFractionView.draw();
		this.popMatrix();

		this.pushMatrix();
		this.translate(0, height-300);
		secondFractionView.draw();
		this.popMatrix();

	}
	
	@Override
	public void keyReleased() {
		// TODO Auto-generated method stub
		super.keyReleased();
		switch (key)
		{
		case('a'):
			this.firstFractionView.multiplier -= 1;
			break;
		case('d'):
			this.firstFractionView.multiplier += 1;
			break;
		case('s'):
			if (this.firstFractionView.common)
				this.firstFractionView.added = !this.firstFractionView.added;
			break;
		case('j'):
			this.secondFractionView.multiplier -= 1;
			break;
		case('l'):
			this.secondFractionView.multiplier += 1;
			break;
		case('k'):
			if (this.secondFractionView.common)
				this.secondFractionView.added = !this.secondFractionView.added;
			break;
		case(' '):
			int d1,n1,d2,n2;
			d1 = (int)this.random(2.0f, 9.0f);
			n1 = (int)this.random(1.0f, d1/2.0f);
			d2 = (int)this.random(2.0f, 9.0f);
			n2 = (int)this.random(1.0f, d2/2.0f);
			
			this.firstFraction = new Fraction(n1,d1);
			this.secondFraction = new Fraction(n2,d2);

			lcd = (int) (firstFraction.denominator() * secondFraction.denominator() / 
					Fraction.gcd(firstFraction.denominator(), secondFraction.denominator()));

			this.firstFractionView.fraction = this.firstFraction;
			this.secondFractionView.fraction = this.secondFraction;
			
			this.firstFractionView.lcd = this.secondFractionView.lcd = this.lcd;
			
			this.firstFractionView.multiplier = this.secondFractionView.multiplier = 1;
		}
		this.firstFractionView.multiplier = PApplet.constrain(this.firstFractionView.multiplier, 1, 20);
		this.secondFractionView.multiplier = PApplet.constrain(this.secondFractionView.multiplier, 1, 20);
		this.redraw();
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		PApplet.main(new String[] {"--present","--bgcolor=#333333","org.q2l.smallab.PartsOfWholeDemo"});
	}

}
